# -*- coding: utf-8 -*-
from django.conf import settings as django_settings
import django


CACHE_LOADER_NAME = 'amp_tools.loader.CachedLoader'
AMP_TOOLS_LOADER = 'amp_tools.loader.Loader'


class SettingsProxy(object):
    def __init__(self, settings, defaults):
        self.settings = settings
        self.defaults = defaults

    def __getattr__(self, attr):
        try:
            return getattr(self.settings, attr)
        except AttributeError:
            try:
                return getattr(self.defaults, attr)
            except AttributeError:
                raise AttributeError(u'settings object has no attribute "%s"' % attr)


class defaults(object):

    AMP_TOOLS_TEMPLATE_FOLDER = u'amp'
    AMP_TOOLS_TEMPLATE_PREFIX = ''
    AMP_TOOLS_GET_PARAMETER = 'amp-content'
    AMP_TOOLS_GET_VALUE = 'amp'
    AMP_TOOLS_ACTIVE_URLS = []


    AMP_TOOLS_TEMPLATE_LOADERS = []

    if django.VERSION[0] < 2 and django.VERSION[1] < 8:
        TEMPLATES = django_settings.TEMPLATE_LOADERS
    else:
        TEMPLATES = django_settings.TEMPLATES[0]['OPTIONS']['loaders']

    for loader in TEMPLATES:
        if isinstance(loader, (tuple, list)) and loader[0] == CACHE_LOADER_NAME:
            for cached_loader in loader[1]:
                if cached_loader != AMP_TOOLS_LOADER:
                    AMP_TOOLS_TEMPLATE_LOADERS.append(cached_loader)
        elif loader != AMP_TOOLS_LOADER:
            AMP_TOOLS_TEMPLATE_LOADERS.append(loader)
    AMP_TOOLS_TEMPLATE_LOADERS = tuple(AMP_TOOLS_TEMPLATE_LOADERS)

settings = SettingsProxy(django_settings, defaults)



"""Create a development and testing environment using a virtualenv."""

from __future__ import unicode_literals
import os
import subprocess
import sys

if sys.version_info[0] >= 3:
    VENV_NAME = '.venv3'
else:
    VENV_NAME = '.venv'

TESTS = os.path.abspath(os.path.dirname(__file__))
REQUIREMENTS = os.path.join(TESTS, 'requirements.pip')
WITH_VENV = os.path.join(TESTS, 'with_venv.sh')
VENV = os.path.abspath(os.path.join(TESTS, '..', VENV_NAME))


def call(*args):
    """Simple ``subprocess.call`` wrapper."""
    if subprocess.call(args):
        raise SystemExit('Error running {0}.'.format(args))


def pip_install(*args):
    """Install packages using pip inside the virtualenv."""
    call(WITH_VENV, VENV_NAME, 'pip', 'install', *args)


if __name__ == '__main__':
    call('virtualenv', '--distribute', '-p', sys.executable, VENV)
    pip_install('-r', REQUIREMENTS)


from __future__ import unicode_literals

import re

from django import template
from django.contrib.sites.models import Site
from django.http import QueryDict
from django.utils.safestring import mark_safe
# from django.utils.encoding import force_text
from django.template import Node, Variable
from django.template.defaultfilters import stringfilter

register = template.Library()

from amp_tools.settings import settings

# For the full HTML element - <img src="image.jpg">
RE_IMG = re.compile('(<img.+?src=["\'].+?["\'].+?>)')

# For the image url itself - "image.jpg"
RE_IMG_SRC = re.compile('src=["\'](.+?)["\']')



@register.simple_tag
def amp_canonical_link(request):
    getvars = request.GET.copy()
    rel = "amphtml"
    if settings.AMP_TOOLS_GET_PARAMETER in getvars:
        del getvars[settings.AMP_TOOLS_GET_PARAMETER]
        rel = "canonical"
    else:
        getvars[settings.AMP_TOOLS_GET_PARAMETER] = settings.AMP_TOOLS_GET_VALUE

    if len(getvars.keys()) > 0:
        getvars = getvars.urlencode()
    else:
        getvars = ''

    href = '%s?%s' % (request.path, getvars) if getvars else request.path

    href = "%s://%s%s" % (request.scheme, Site.objects.get_current().domain, href)

    return mark_safe('<link rel="%s" href="%s" />' % (rel, href))


class AddGetParameter(Node):
    def __init__(self, values, url=None):
        self.url = url
        self.values = values

    def render(self, context):
        if self.url:
            params = QueryDict(self.values)
        else:
            req = Variable('request').resolve(context)
            self.url = req.path
            params = req.GET.copy()

            for key, value in self.values.items():
                resolved = value.resolve(context)
                if resolved:
                    params[key] = value.resolve(context)

        return "{}?{}".format(self.url, params.urlencode())


@register.tag
def amp_link(parser, token):
    try:
        # split_contents() knows not to split quoted strings.
        tag_name, url = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires a single argument" % token.contents.split()[0]
        )
    if not (url[0] == url[-1] and url[0] in ('"', "'")):
        raise template.TemplateSyntaxError(
            "%r tag's argument should be in quotes" % tag_name
        )
    params = "{}={}".format(
        settings.AMP_TOOLS_GET_PARAMETER,
        settings.AMP_TOOLS_GET_VALUE
    )
    return AddGetParameter(params, url[1:-1])


@register.filter
def amp_urlparam(value):
    return "%s?%s=%s" % (value, settings.AMP_TOOLS_GET_PARAMETER, settings.AMP_TOOLS_GET_VALUE)


@register.filter(name='amp_img')
@stringfilter
def amp_img(html_code):
    """Convert <img> to <amp-img>"""
    img_elements = RE_IMG.findall(html_code)
    for img_el in img_elements:
        replace_str = img_el.replace('/>','>').replace('>', ' layout="responsive"></amp-img>')
        html_code = html_code.replace(img_el, replace_str)
    return html_code.replace('<img', '<amp-img')



import re

from amp_tools.settings import settings
from amp_tools import set_amp_detect


class AMPDetectionMiddleware(object):

    def __init__(self, get_response=None):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        return response

    def process_request(self, request):
        if settings.AMP_TOOLS_GET_PARAMETER in request.GET:
            if request.GET[settings.AMP_TOOLS_GET_PARAMETER] == settings.AMP_TOOLS_GET_VALUE:
                if settings.AMP_TOOLS_ACTIVE_URLS:
                    for url in settings.AMP_TOOLS_ACTIVE_URLS:
                        if not isinstance(url, re._pattern_type):
                            url = str(url)
                        url_re = re.compile(url)

                        if url_re.match(request.path_info):
                            set_amp_detect(is_amp_detect=True, request=request)
                else:
                    set_amp_detect(is_amp_detect=True, request=request)
        else:
            set_amp_detect(is_amp_detect=False, request=request)



from __future__ import unicode_literals

import threading

from django.test import TestCase, override_settings
from django.template import Template, RequestContext


from mock import MagicMock, Mock, patch, call

from amp_tools import get_amp_detect
from amp_tools.middleware import AMPDetectionMiddleware
from amp_tools.settings import settings as amp_setting
from amp_tools.templatetags.amp_tags import amp_img



def _reset():
    '''
    Reset the thread local.
    '''
    import amp_tools
    del amp_tools._local
    amp_tools._local = threading.local()


class BaseTestCase(TestCase):
    def setUp(self):
        _reset()

    def tearDown(self):
        _reset()


class DetectAMPMiddlewareTests(BaseTestCase):

    def setUp(self):
        self.amp_get_parameter = amp_setting.AMP_TOOLS_GET_PARAMETER
        self.amp_get_value = amp_setting.AMP_TOOLS_GET_VALUE

    def test_default_page(self):
        request = Mock()
        request.META = MagicMock()
        request.GET = {}
        middleware = AMPDetectionMiddleware()
        middleware.process_request(request)
        # default flavour is set
        self.assertEqual(get_amp_detect(), '')

    @patch('amp_tools.middleware.set_amp_detect')
    def test_set_amp_detect_through_get_parameter(self, set_amp_detect):
        request = Mock()
        request.META = MagicMock()
        request.GET = {'amp-content': 'amp'}
        middleware = AMPDetectionMiddleware()
        middleware.process_request(request)
        self.assertEqual(set_amp_detect.call_args, call(is_amp_detect=True, request=request))

    def test_tamplate_tags(self):
        request = Mock()
        request.META = MagicMock()
        request.GET = {'amp-content': 'amp'}
        template = Template(
            '{% load amp_tags %}{% amp_link "/path/" %}'
        )
        context = RequestContext(request, {},)

        rendered = template.render(context)

        self.assertEqual(
            rendered,
            "/path/?%s=%s" % (self.amp_get_parameter, self.amp_get_value)
        )

        html_content = """
            <html><body>
                <img alt="alternate text" src="/media/uploads/img.png" style="width: 100%;">
                <img alt="alternate text2" src="/media/uploads/img2.png" style="width: 100%;" />
            </body></html>
        """
        amp_content = amp_img(html_content)
        self.assertNotEqual(amp_content, html_content)
        self.assertEqual(amp_content, """
            <html><body>
                <amp-img alt="alternate text" src="/media/uploads/img.png" style="width: 100%;" layout="responsive"></amp-img>
                <amp-img alt="alternate text2" src="/media/uploads/img2.png" style="width: 100%;"  layout="responsive"></amp-img>
            </body></html>
        """)


    @patch('amp_tools.middleware.set_amp_detect')
    @override_settings(AMP_TOOLS_ACTIVE_URLS=['^/$'])
    def test_set_amp_not_set_url_allowed(self, set_amp_detect):
        request = Mock()
        request.META = MagicMock()
        request.GET = {'amp-content': 'amp'}
        request.path_info = '/'
        middleware = AMPDetectionMiddleware()
        middleware.process_request(request)
        self.assertEqual(set_amp_detect.call_args, call(is_amp_detect=True, request=request))

    @patch('amp_tools.middleware.set_amp_detect')
    @override_settings(AMP_TOOLS_ACTIVE_URLS=['^/$'])
    def test_set_amp_not_set_url_not_allowed(self, set_amp_detect):
        request = Mock()
        request.META = MagicMock()
        request.GET = {'amp-content': 'amp'}
        request.path_info = '/not-amp-url/'
        middleware = AMPDetectionMiddleware()
        middleware.process_request(request)
        self.assertEqual(0, len(set_amp_detect.call_args_list))




# -*- coding: utf-8 -*-

import threading
from amp_tools.settings import settings

__version__ = '0.2.0'

_local = threading.local()


def set_amp_detect(is_amp_detect=False, request=None):
    request = request or getattr(_local, 'request', None)
    if request:
        request.is_amp_detect = is_amp_detect
    _local.is_amp_detect = is_amp_detect


def get_amp_detect(request=None):
    is_amp_detect = False
    request = request or getattr(_local, 'request', None)

    # check if is_amp_detect is set on request
    if not is_amp_detect and hasattr(request, 'is_amp_detect'):
        is_amp_detect = request.is_amp_detect

    # if set out of a request-response cycle its stored on the thread local
    if not is_amp_detect:
        is_amp_detect = getattr(_local, 'is_amp_detect', False)
    return settings.AMP_TOOLS_TEMPLATE_FOLDER if is_amp_detect else u""



#!/usr/bin/env python
import os
import sys

from django.core.management import execute_from_command_line

os.environ['DJANGO_SETTINGS_MODULE'] = 'settings_tests'

if __name__ == '__main__':
    root = os.path.join(os.path.dirname(__file__), '..')
    sys.path.append(os.path.abspath(root))
    execute_from_command_line()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
from distutils.core import setup

PROJECT_NAME = 'amp_tools'
ROOT = os.path.abspath(os.path.dirname(__file__))
VENV = os.path.join(ROOT, '.venv')
VENV_LINK = os.path.join(VENV, 'local')

install_requires = [
    'django>=1.11.0',
]


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


data_files = []
for dirpath, dirnames, filenames in os.walk(PROJECT_NAME):
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'):
            del dirnames[i]
    if '__init__.py' in filenames:
        continue
    elif filenames:
        for f in filenames:
            data_files.append(os.path.join(
                dirpath[len(PROJECT_NAME) + 1:], f))


def read(filename):
    return open(os.path.join(ROOT, filename)).read()


class VenvLinkDeleted(object):

    restore_link = False

    def __enter__(self):
        """Remove the link."""
        if os.path.islink(VENV_LINK):
            os.remove(VENV_LINK)
            self.restore_link = True

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Restore the link."""
        if self.restore_link:
            os.symlink(VENV, VENV_LINK)

with VenvLinkDeleted():
    setup(
        name='django-amp-tools',
        version=get_version('amp_tools'),
        packages=[
            PROJECT_NAME,
            '{0}.templatetags'.format(PROJECT_NAME),
        ],
        package_data={PROJECT_NAME: data_files},
        include_package_data=True,
        license='MIT License',
        description='Accelerated mobile pages (AMP) in django.',
        keywords='django AMP accelerated',
        long_description=read('README.rst'),
        url='http://github.com/shtalinberg/django-amp-tools',
        author='Oleksandr Shtalinberg',
        author_email='O.Shtalinberg@gmail.com',
        classifiers=[
            'Development Status :: 4 - Beta',
            'Environment :: Web Environment',
            'Framework :: Django',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',  # example license
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            # Replace these appropriately if you are stuck on Python 2.
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.6',
            'Topic :: Internet :: WWW/HTTP',
            'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        ],
        install_requires=install_requires,
        zip_safe=False,
    )
